/**
 * Copyright 2009-2012 Bull S.A.S.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.ow2.util.ee.metadata.ejbjar.impl;

import java.util.Arrays;
import java.util.Collection;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.ow2.util.ee.deploy.api.deployable.EJBDeployable;
import org.ow2.util.ee.deploy.impl.deployable.metadata.DeployableMetadata;
import org.ow2.util.ee.metadata.common.api.struct.IJInterceptors;
import org.ow2.util.ee.metadata.ejbjar.api.IEjbJarClassMetadata;
import org.ow2.util.ee.metadata.ejbjar.api.IEjbJarDeployableMetadata;
import org.ow2.util.ee.metadata.ejbjar.api.IEjbJarFieldMetadata;
import org.ow2.util.ee.metadata.ejbjar.api.IEjbJarMethodMetadata;
import org.ow2.util.ee.metadata.ejbjar.api.struct.IApplicationException;
import org.ow2.util.ee.metadata.ejbjar.api.struct.IJCommonBean;
import org.ow2.util.ee.metadata.ejbjar.api.xml.struct.IEJB3;
import org.ow2.util.ee.metadata.ejbjar.impl.struct.JApplicationException;
import org.ow2.util.ee.metadata.ws.api.xml.struct.IWebservices;
import org.ow2.util.log.Log;
import org.ow2.util.log.LogFactory;
import org.ow2.util.marshalling.Serialization;
import org.ow2.util.marshalling.SerializationException;

/**
 * This class represents the metadata of all classes of an EjbJar file. From
 * this class, we can get metadata of all beans.
 * @param <E> an implementation of EJBDeployable
 * @param <D> an implementation of IEjbJarDeployableMetadata
 * @param <C> an implementation of IEjbJarClassMetadata
 * @param <M> an implementation of IEjbJarMethodMetadata
 * @param <F> an implementation of IEjbJarFieldMetadata
 * @author Florent Benoit
 */
public class EjbJarDeployableMetadata<
    E extends EJBDeployable<E>,
    D extends IEjbJarDeployableMetadata<E, D, C, M, F>,
    C extends IEjbJarClassMetadata<E, D, C, M, F>,
    M extends IEjbJarMethodMetadata<E, D, C, M, F>,
    F extends IEjbJarFieldMetadata<E, D, C, M, F>>
        extends DeployableMetadata<E> implements IEjbJarDeployableMetadata<E, D, C, M, F> {

    /**
     * Serial version UID.
     */
    private static final long serialVersionUID = 2467337105092579966L;

    /**
     * Logger.
     */
    private static Log logger = LogFactory.getLog(EjbJarDeployableMetadata.class);

    /**
     * List of class annotations metadata.
     */
    private Map<String, C> classesAnnotationMetadata = null;

    /**
     * Map between ejb-name and a map of Classname, ClassAnnotationMetadata.
     */
    private Map<String, Map<String, C>> beanClassesAnnotationMetadata = null;

    /**
     * Link to the Deployment Descriptor object.
     */
    private IEJB3 ejb3 = null;

    /**
     * Link to the Webservices.
     */
    private IWebservices webservices = null;

    /**
     * List of application exceptions used on this ejb-jar.
     */
    private Map<String, IApplicationException> applicationExceptions = null;

    /**
     * List of default interceptors classes.
     */
    private IJInterceptors defaultInterceptorsClasses = null;

    /**
     * No-arg Constructor.
     */
    public EjbJarDeployableMetadata() {
        super();
        this.classesAnnotationMetadata = new HashMap<String, C>();
        this.beanClassesAnnotationMetadata = new HashMap<String, Map<String, C>>();
    }

    /**
     * Constructor.
     * @param deployable the ejb-jar deployable
     */
    public EjbJarDeployableMetadata(final E deployable) {
        this();
        setDeployable(deployable);
    }

    /**
     * Add annotation metadata for a given class.
     * @param classAnnotationMetadata annotation metadata of a class.
     */
    public void addScannedClassMetadata(final C classAnnotationMetadata) {
        String key = classAnnotationMetadata.getClassName();
        // already exists ?
        if (classesAnnotationMetadata.containsKey(key)) {
            String msg = logger.getI18n().getMessage("EjbJarAnnotationMetadata.addClassAnnotationMetadata.alreadyPresent", key);
            logger.debug(msg);
            throw new IllegalStateException(msg);
        }
        classesAnnotationMetadata.put(key, classAnnotationMetadata);
    }

    /**
     * Gets the class metadata for the given ejb-name.
     * @return class metadata or null if not found
     * @param ejbName the name of the EJB.
     */
    public C getEjbJarClassMetadataForEjbName(final String ejbName) {
        // First, get the Bean Map
        Map<String, C> beanMapClassMetadata = beanClassesAnnotationMetadata.get(ejbName);

        // Not found
        if (beanMapClassMetadata == null) {
            return null;
        }

        // It has been found, now search the bean class
        for (C metadata : beanMapClassMetadata.values()) {
            IJCommonBean bean = metadata.getJCommonBean();
            if (bean != null && ejbName.equals(bean.getName())) {
                return metadata;
            }
        }
        return null;
    }

    /**
     * Gets the class metadata for the given ejb-name/ejbclass.
     * @return class metadata or null if not found
     * @param ejbName the name of the EJB.
     * @param ejbClass the class of the EJB.
     */
    @SuppressWarnings("unchecked")
    public C getBeanEjbJarClassMetadataForEjbName(final String ejbName, final String ejbClass) {

        logger.debug("Call with ''{0}'' and ejbClass ''{1}''", ejbName, ejbClass);

        // First, get the Bean Map
        Map<String, C> beanMapClassMetadata = beanClassesAnnotationMetadata.get(ejbName);
        // Do not exists yet, creates a new one
        if (beanMapClassMetadata == null) {
            beanMapClassMetadata = new HashMap<String, C>();
            // add it in the Map
            beanClassesAnnotationMetadata.put(ejbName, beanMapClassMetadata);

            logger.debug("Creating a Map for EJB ''{0}''", ejbName);

        }

        // Now, get the class from the previous analysis classes
        String ejbClassEncoded = null;

        // No EJB-Class ?
        if (ejbClass == null) {
            // needs to find a matching class for the given EJB-NAME
            for (C tmpMetaData : getEjbJarClassMetadataCollection()) {
                // Try to see if there is a bean name
                IJCommonBean bean = tmpMetaData.getJCommonBean();
                if (bean != null) {
                    // found !
                    if (ejbName != null && ejbName.equals(bean.getName())) {
                        ejbClassEncoded = tmpMetaData.getClassName();
                        break;
                    }
                }
            }
        } else {
            ejbClassEncoded = encode(ejbClass);
        }


        // Try to see if there is a metadata
        C classMetadata = beanMapClassMetadata.get(ejbClassEncoded);
        if (classMetadata != null) {
            return classMetadata;
        }

        // Not found, get it from the scanned classes
        classMetadata = getScannedClassMetadata(ejbClassEncoded);

        if (classMetadata == null) {
            throw new IllegalStateException("No metadata defined for the EJB class '" + ejbClass + "'");
        }

        // Clone it
        C clonedClassMetadata = null;
        try {
            clonedClassMetadata = Serialization.cloneObject(classMetadata);
        } catch (SerializationException e) {
            throw new IllegalStateException("Cannot clone classmetada for '" + ejbClass + "'", e);
        }
        // This metadata is part of a bean and need to be customized
        clonedClassMetadata.setLinkedBean(ejbName);

        logger.debug("Cloning scanning classmetadata ''{0}''", ejbClassEncoded);

        // Add the cloned classmetadata in the map
        beanMapClassMetadata.put(ejbClassEncoded, clonedClassMetadata);

        // Set its parent to ourself
        clonedClassMetadata.setEjbJarArchiveMetadata((D) this);


        // return object
        return clonedClassMetadata;
    }





    /**
     * Encode a classname (replace . by /) as it's the internal form of the
     * metadata
     * @param className the name of the class to encode
     * @return encoded name
     */
    protected static String encode(final String className) {
        if (className != null) {
            return className.replace(".", "/");
        }
        return null;
    }


    /**
     * Get class annotation metadata.
     * @param className key of the map of annotations bean.
     * @return Bean annotation metadata of a given name or null if class not found.
     */
    public C getScannedClassMetadata(final String className) {
        return classesAnnotationMetadata.get(className);
    }

    /**
     * Get collections of bean annotation metadata.
     * @return collections of bean annotation metadata.
     */
    public Collection<C> getEjbJarClassMetadataCollection() {
        return classesAnnotationMetadata.values();
    }

    /**
     * @return the ejb3 deployment descriptor object.
     */
    public IEJB3 getEjb3() {
        return ejb3;
    }

    /**
     * Sets the ejb3 deployment descriptor object.
     * @param ejb3 the ejb3 deployment descriptor object.
     */
    public void setEjb3(final IEJB3 ejb3) {
        this.ejb3 = ejb3;
    }

    /**
     * @return the webservices 1.2 deployment descriptor object.
     */
    public IWebservices getWebservices12() {
        return webservices;
    }

    /**
     * Sets the webservices 1.2 deployment descriptor object.
     *
     * @param webservices12 the webservices 1.2 deployment descriptor object.
     */
    public void setWebservices12(IWebservices webservices12) {
        this.webservices = webservices12;
    }

    /**
     * Gets the list of application exceptions defined on this ejb jar metadata.
     * @return the list of application exceptions defined on this ejb jar
     *         metadata.
     */
    public Map<String, IApplicationException> getApplicationExceptions() {
        if (applicationExceptions != null) {
            return applicationExceptions;
        }

        // compute it
        applicationExceptions = new HashMap<String, IApplicationException>();

        // For each class, look if it is an application exception
        for (C classMetadata : getEjbJarClassMetadataCollection()) {
            IApplicationException appException = classMetadata.getApplicationException();
            // found it then add it in the map.
            if (appException != null) {
                applicationExceptions.put(classMetadata.getClassName().replaceAll("/", "."), appException);
                logger.debug("Adding application exception ''{}''", classMetadata.getClassName().replaceAll("/", "."));
            }
        }

        // Add a default application exception (for checked exception)
        applicationExceptions.put("DEFAULT", new JApplicationException());

        return applicationExceptions;
    }

    /**
     * @return object representing list of default interceptor classes.
     */
    public IJInterceptors getDefaultInterceptorsClasses() {
        return defaultInterceptorsClasses;
    }

    /**
     * Sets the object representing the default interceptor classes.
     * @param defaultInterceptorsClasses list of classes
     */
    public void setDefaultInterceptorsClasses(final IJInterceptors defaultInterceptorsClasses) {
        this.defaultInterceptorsClasses = defaultInterceptorsClasses;
    }

    /**
     * @return a list of all beans that are defined
     */
    public List<String> getBeanNames() {
        
        return Arrays.asList(beanClassesAnnotationMetadata.keySet().toArray(new String[0]));
    }

    /**
     * @param beanName the bean that will be used in order to get the classes.
     * @return a list of all classes that are for a given bean
     */
    @SuppressWarnings("unchecked")
    public Collection<C> getClassesForBean(final String beanName) {
        Map<String, C> beanClasses = beanClassesAnnotationMetadata.get(beanName);
        if (beanClasses == null) {
            return Collections.EMPTY_LIST;
        }
        return beanClasses.values();
    }

    /**
     * @param beanName the bean that will be used in order to get the classes.
     * @return a list of all classes that are for a given bean
     */
    public C getClassForBean(final String beanName, final String className) {
        return beanClassesAnnotationMetadata.get(beanName).get(className);
    }

    /**
     * @param beanName the bean that will be used in order to get the classes.
     * @return a list of all classes that are for a given bean
     */
    public List<String> getClassesnameForBean(final String beanName) {
        return Arrays.asList(beanClassesAnnotationMetadata.get(beanName).keySet().toArray(new String[0]));
    }




}
