/**
 * OW2 Utilities
 * Copyright (C) 2009-2012 Bull S.A.S.
 * Contact: utilities@ow2.org
 *
 * Licensed  under the  Apache License,  Version 2.0  (the "License");
 * you may not use  this file  except in  compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed  under the  License is distributed on an "AS IS" BASIS,
 * WITHOUT  WARRANTIES OR CONDITIONS  OF ANY KIND, either  express  or
 * implied.
 *
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.ow2.util.ee.metadata.ejbjar.impl.configurator.visitor;

import java.util.concurrent.TimeUnit;

import org.ow2.util.ee.metadata.common.impl.configurator.visitor.AbsAnnotationVisitor;
import org.ow2.util.ee.metadata.common.impl.configurator.visitor.AnnotationType;
import org.ow2.util.ee.metadata.ejbjar.api.interfaces.IEjbStatefulTimeout;
import org.ow2.util.ee.metadata.ejbjar.api.struct.IJEjbStatefulTimeout;
import org.ow2.util.ee.metadata.ejbjar.impl.struct.JEJBStatefulTimeout;

/**
 * This class manages the handling of &#64;{@link javax.ejb.StatefulTimeout}
 * annotation.
 * @param <T> An implementation of IEjbStatefulTimeout interface.
 * @author Loic Albertin
 */
public class JavaxEjbStatefulTimeoutVisitor<T extends IEjbStatefulTimeout> extends AbsAnnotationVisitor<T> implements
        AnnotationType {

    /**
     * Type of annotation.
     */
    public static final String TYPE = "Ljavax/ejb/StatefulTimeout;";

    /**
     * value attribute of the annotation.
     */
    private static final String VALUE = "value";


    /**
     * unit attribute of the annotation.
     */
    private static final String UNIT = "unit";


    /**
     * Access Timeout information.
     */
    private IJEjbStatefulTimeout jEjbStatefulTimeout = null;

    /**
     * Constructor.
     * @param annotationMetadata linked to a class or method or field metadata
     */
    public JavaxEjbStatefulTimeoutVisitor(final T annotationMetadata) {
        super(annotationMetadata);
        jEjbStatefulTimeout = new JEJBStatefulTimeout();
    }

    /**
     * Visits a primitive value of the annotation.<br>
     * @param name the value name.
     * @param value the actual value, whose type must be {@link Byte},
     *        {@link Boolean}, {@link Character}, {@link Short},
     *        {@link Integer}, {@link Long}, {@link Float}, {@link Double},
     *        {@link String} or {@link org.ow2.util.scan.api.IType}.
     */
    @Override
    public void visit(final String name, final Object value) {
        if (VALUE.equals(name)) {
            jEjbStatefulTimeout.setValue((Long) value);
        }
    }

    /**
     * Visits an enumeration value of the annotation.
     * @param name the value name.
     * @param desc the class descriptor of the enumeration class.
     * @param value the actual enumeration value.
     */
    @Override
    public void visitEnum(final String name, final String desc, final String value) {
       if (UNIT.equals(name)) {
           jEjbStatefulTimeout.setUnit(TimeUnit.valueOf(value.toUpperCase()));
        }
    }

    /**
     * Visits the end of the annotation.<br>
     * Creates the object and store it.
     */
    @Override
    public void visitEnd() {
        // set flag on field
        getAnnotationMetadata().setJavaxEjbStatefulTimeout(jEjbStatefulTimeout);
    }

    /**
     * @return type of the annotation (its description)
     */
    public String getType() {
        return TYPE;
    }

}
