/**
 * Copyright 2009-2012 Bull S.A.S.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.ow2.util.ee.metadata.ejbjar.impl.configurator.visitor.clazz;

import static org.ow2.util.ee.metadata.ejbjar.api.ClassType.MDB;

import org.ow2.util.ee.deploy.api.deployable.EJBDeployable;
import org.ow2.util.ee.metadata.common.impl.configurator.visitor.AnnotationType;
import org.ow2.util.ee.metadata.ejbjar.api.IEjbJarClassMetadata;
import org.ow2.util.ee.metadata.ejbjar.api.IEjbJarDeployableMetadata;
import org.ow2.util.ee.metadata.ejbjar.api.IEjbJarFieldMetadata;
import org.ow2.util.ee.metadata.ejbjar.api.IEjbJarMethodMetadata;
import org.ow2.util.ee.metadata.ejbjar.api.struct.IJMessageDriven;
import org.ow2.util.ee.metadata.ejbjar.impl.struct.JActivationConfigProperty;
import org.ow2.util.ee.metadata.ejbjar.impl.struct.JMessageDriven;
import org.ow2.util.scan.api.DefaultAnnotationVisitor;
import org.ow2.util.scan.api.IAnnotationVisitor;
import org.ow2.util.scan.api.IType;

/**
 * This class manages the handling of &#64;{@link javax.ejb.MessageDriven}
 * annotation.
 * @param <E> an implementation of EJBDeployable
 * @param <D> an implementation of IEjbJarDeployableMetadata
 * @param <C> an implementation of IEjbJarClassMetadata
 * @param <M> an implementation of IEjbJarMethodMetadata
 * @param <F> an implementation of IEjbJarFieldMetadata
 * @author Florent Benoit
 */
public class JavaxEjbMessageDrivenVisitor<
    E extends EJBDeployable<E>,
    D extends IEjbJarDeployableMetadata<E, D, C, M, F>,
    C extends IEjbJarClassMetadata<E, D, C, M, F>,
    M extends IEjbJarMethodMetadata<E, D, C, M, F>,
    F extends IEjbJarFieldMetadata<E, D, C, M, F>>
        extends AbsCommonEjbVisitor<IJMessageDriven, E, D, C, M, F> implements AnnotationType {

    /**
     * Type of annotation.
     */
    public static final String TYPE = "Ljavax/ejb/MessageDriven;";

    /**
     * messageListenerInterface attribute of the annotation.
     */
    private static final String MESSAGE_LISTENER_INTERFACE = "messageListenerInterface";

    /**
     * activationConfig attribute of the annotation.
     */
    private static final String ACTIVATION_CONFIG = "activationConfig";

    /**
     * Message listener Interface.
     */
    private String messageListenerInterface = null;

    /**
     * Constructor.
     * @param classAnnotationMetadata linked to a class metadata
     */
    public JavaxEjbMessageDrivenVisitor(final C classAnnotationMetadata) {
        super(classAnnotationMetadata);
    }

    /**
     * Visits a primitive value of the annotation.<br>
     * @param name the value name.
     * @param value the actual value, whose type must be {@link Byte},
     *        {@link Boolean}, {@link Character}, {@link Short},
     *        {@link Integer}, {@link Long}, {@link Float}, {@link Double},
     *        {@link String} or {@link IType}.
     */
    @Override
    public void visit(final String name, final Object value) {
        super.visit(name, value);
        if (name.equals(MESSAGE_LISTENER_INTERFACE)) {
            this.messageListenerInterface = ((IType) value).getInternalName();
        }
    }

    /**
     * Visits an enumeration value of the annotation.
     * @param name the value name.
     * @param desc the class descriptor of the enumeration class.
     * @param value the actual enumeration value.
     */
    @Override
    public void visitEnum(final String name, final String desc, final String value) {

    }

    /**
     * Visits a nested annotation value of the annotation.
     * @param name the value name.
     * @param desc the class descriptor of the nested annotation class.
     * @return a non null visitor to visit the actual nested annotation value.
     *         <i>The nested annotation value must be fully visited before
     *         calling other methods on this annotation visitor</i>.
     */
    @Override
    public IAnnotationVisitor visitAnnotation(final String name, final String desc) {
        return this;
    }

    /**
     * Visits an array value of the annotation.
     * @param name the value name.
     * @return a non null visitor to visit the actual array value elements. The
     *         'name' parameters passed to the methods of this visitor are
     *         ignored. <i>All the array values must be visited before calling
     *         other methods on this annotation visitor</i>.
     */
    @Override
    public IAnnotationVisitor visitArray(final String name) {
        if (name.equals(ACTIVATION_CONFIG)) {
            return new ActivationConfigPropertyVisitor();
        }
        return this;
    }

    /**
     * Visits the end of the annotation. Creates the object and store it
     */
    @Override
    public void visitEnd() {
        super.visitEnd();

        // message listener interface
        getJCommonBean().setMessageListenerInterface(this.messageListenerInterface);

        // Set type
        getAnnotationMetadata().setClassType(MDB);

    }

    /**
     * @return type of the annotation (its description)
     */
    public String getType() {
        return TYPE;
    }

    /**
     * Classes manages the parsing of activationConfig[] array of &#64;{@link javax.ejb.MessageDriven}
     * annotation.
     * @author Florent Benoit
     */
    class ActivationConfigPropertyVisitor extends DefaultAnnotationVisitor {

        /**
         * Attribute for property name.
         */
        private static final String PROPERTY_NAME = "propertyName";

        /**
         * Attribute for property value.
         */
        private static final String PROPERTY_VALUE = "propertyValue";

        /**
         * Property name.
         */
        private String propertyName = null;

        /**
         * Property value.
         */
        private String propertyValue = null;

        /**
         * Visits a primitive value of the annotation.
         * @param name the value name.
         * @param value the actual value, whose type must be {@link Byte},
         *        {@link Boolean}, {@link Character}, {@link Short},
         *        {@link Integer}, {@link Long}, {@link Float},
         *        {@link Double}, {@link String} or {@link IType}.
         */
        @Override
        public void visit(final String name, final Object value) {
            if (name.equals(PROPERTY_NAME)) {
                this.propertyName = (String) value;
            } else if (name.equals(PROPERTY_VALUE)) {
                this.propertyValue = (String) value;
            }
        }

        /**
         * Visits the end of the annotation. Creates the object and store it
         */
        @Override
        public void visitEnd() {
            // Add an activation property
            JActivationConfigProperty jActivationConfigProperty = new JActivationConfigProperty(this.propertyName,
                    this.propertyValue);
            getJCommonBean().addActivationConfigProperty(jActivationConfigProperty);

        }

    }

    /**
     * @return the object representing common bean.
     */
    @Override
    public IJMessageDriven getJCommonBean() {
        IJMessageDriven jMessageDriven = getAnnotationMetadata().getJMessageDriven();
        if (jMessageDriven == null) {
            jMessageDriven = new JMessageDriven();
            getAnnotationMetadata().setJMessageDriven(jMessageDriven);
        }
        return jMessageDriven;
    }

}
