/**
 * Copyright 2009-2012 Bull S.A.S.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.ow2.util.ee.metadata.ejbjar.impl.configurator.visitor.method;

import org.ow2.util.ee.metadata.common.impl.configurator.visitor.AbsAnnotationVisitor;
import org.ow2.util.ee.metadata.common.impl.configurator.visitor.AnnotationType;
import org.ow2.util.ee.metadata.ejbjar.api.interfaces.IEjbSchedule;
import org.ow2.util.ee.metadata.ejbjar.impl.struct.JEJBSchedule;
import org.ow2.util.scan.api.IType;

/**
 * This class manages the handling of &#64;{@link javax.ejb.Schedule} annotation.
 * @param <T> An implementation of IAnnotationEJB interface.
 * @author Florent Benoit
 */
public class JavaxEjbScheduleVisitor<T extends IEjbSchedule>
          extends AbsAnnotationVisitor<T> implements AnnotationType {

    /**
     * second attribute of the annotation.
     */
    private static final String SECOND = "second";

    /**
     * minute attribute of the annotation.
     */
    private static final String MINUTE = "minute";

    /**
     * hour attribute of the annotation.
     */
    private static final String HOUR = "hour";

    /**
     * dayOfMonth attribute of the annotation.
     */
    private static final String DAY_OF_MONTH = "dayOfMonth";

    /**
     * month attribute of the annotation.
     */
    private static final String MONTH = "month";

    /**
     * dayOfWeek attribute of the annotation.
     */
    private static final String DAY_OF_WEEK = "dayOfWeek";

    /**
     * year attribute of the annotation.
     */
    private static final String YEAR = "year";

    /**
     * timezone attribute of the annotation.
     */
    private static final String TIMEZONE = "timezone";

    /**
     * info attribute of the annotation.
     */
    private static final String INFO = "info";

    /**
     * persistent attribute of the annotation.
     */
    private static final String PERSISTENT = "persistent";


    /**
     * Type of annotation.
     */
    public static final String TYPE = "Ljavax/ejb/Schedule;";

    /**
     * Internal object used representing &#64;{@link javax.ejb.Schedule} annotation.
     */
    private JEJBSchedule jEjbSchedule = null;

    /**
     * Constructor.
     * @param annotationMetadata linked to a class or method metadata
     */
    public JavaxEjbScheduleVisitor(final T annotationMetadata) {
        super(annotationMetadata);
        this.jEjbSchedule = new JEJBSchedule();
    }


    /**
     * Visits a primitive value of the annotation.<br>
     * @param name the value name.
     * @param value the actual value, whose type must be {@link Byte},
     *        {@link Boolean}, {@link Character}, {@link Short},
     *        {@link Integer}, {@link Long}, {@link Float}, {@link Double},
     *        {@link String} or {@link org.ow2.util.scan.api.IType}.
     */
    @Override
    public void visit(final String name, final Object value) {
        if (SECOND.equals(name)) {
            jEjbSchedule.setSecond((String) value);
        } else if (MINUTE.equals(name)) {
            jEjbSchedule.setMinute((String) value);
        } else if (HOUR.equals(name)) {
            jEjbSchedule.setHour((String) value);
        } else if (DAY_OF_MONTH.equals(name)) {
            jEjbSchedule.setDayOfMonth((String) value);
        } else if (MONTH.equals(name)) {
            jEjbSchedule.setMonth((String) value);
        } else if (DAY_OF_WEEK.equals(name)) {
            jEjbSchedule.setDayOfWeek((String) value);
        } else if (YEAR.equals(name)) {
            jEjbSchedule.setYear((String) value);
        } else if (TIMEZONE.equals(name)) {
            jEjbSchedule.setTimezone((String) value);
        } else if (INFO.equals(name)) {
            jEjbSchedule.setInfo((String) value);
        } else if (PERSISTENT.equals(name)) {
            jEjbSchedule.setPersistent((Boolean) value);
        }
    }

    /**
     * Visits the end of the annotation. <br>
     * Creates the object and store it.
     */
    @Override
    public void visitEnd() {
        // add object
        getAnnotationMetadata().addJavaxEjbSchedule(jEjbSchedule);
    }

    /**
     * @return type of the annotation (its description)
     */
    public String getType() {
        return TYPE;
    }


    /**
     * @return Internal object used representing &#64;{@link javax.ejb.Schedule} annotation.
     */
    protected JEJBSchedule getJEJBSchedule() {
        return jEjbSchedule;
    }

    /**
     * Sets the jEjbSchedule object.
     * @param jEjbSchedule the object which replaced the previous one.
     */
    protected void setJEJBSchedule(final JEJBSchedule jEjbSchedule) {
        this.jEjbSchedule = jEjbSchedule;
    }

}
