/**
 * Copyright 2009-2012 Bull S.A.S.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.ow2.util.ee.metadata.ejbjar.impl.configurator.visitor.method;

import org.ow2.util.ee.deploy.api.deployable.EJBDeployable;
import org.ow2.util.ee.metadata.common.api.struct.IJavaxJwsWebMethod;
import org.ow2.util.ee.metadata.common.impl.configurator.visitor.AbsAnnotationVisitor;
import org.ow2.util.ee.metadata.common.impl.configurator.visitor.AnnotationType;
import org.ow2.util.ee.metadata.common.impl.struct.JwsWebMethod;
import org.ow2.util.ee.metadata.ejbjar.api.IEjbJarClassMetadata;
import org.ow2.util.ee.metadata.ejbjar.api.IEjbJarDeployableMetadata;
import org.ow2.util.ee.metadata.ejbjar.api.IEjbJarFieldMetadata;
import org.ow2.util.ee.metadata.ejbjar.api.IEjbJarMethodMetadata;

/**
 * This class manages the handling of &#64;{@link javax.ejb.PostActivate} annotation.
 * @param <E> an implementation of EJBDeployable
 * @param <D> an implementation of IEjbJarDeployableMetadata
 * @param <C> an implementation of IEjbJarClassMetadata
 * @param <M> an implementation of IEjbJarMethodMetadata
 * @param <F> an implementation of IEjbJarFieldMetadata
 * @author Florent Benoit
 */
public class JavaxJwsWebMethodVisitor<
    E extends EJBDeployable<E>,
    D extends IEjbJarDeployableMetadata<E, D, C, M, F>,
    C extends IEjbJarClassMetadata<E, D, C, M, F>,
    M extends IEjbJarMethodMetadata<E, D, C, M, F>,
    F extends IEjbJarFieldMetadata<E, D, C, M, F>>
        extends AbsAnnotationVisitor<M>
        implements AnnotationType {

    /**
     * Type of annotation.
     */
    public static final String TYPE = "Ljavax/jws/WebMethod;";

    /**
     * Action attribute of the annotation.
     */
    private static final String ACTION = "action";

    /**
     * Exclude attribute of the annotation.
     */
    private static final String EXCLUDE = "exclude";

    /**
     * Operation name attribute of the annotation.
     */
    private static final String OPERATION_NAME = "operationName";

    /**
     * Annotation struct.
     */
    private IJavaxJwsWebMethod javaxJwsWebMethod = null;

    /**
     * Constructor.
     * @param methodAnnotationMetadata linked to a method metadata
     */
    public JavaxJwsWebMethodVisitor(final M methodAnnotationMetadata) {
        super(methodAnnotationMetadata);
        this.javaxJwsWebMethod = new JwsWebMethod();
    }

    /**
     * Visits a primitive value of the annotation.<br>
     * @param name the value name.
     * @param value the actual value, whose type must be {@link Byte},
     *        {@link Boolean}, {@link Character}, {@link Short},
     *        {@link Integer}, {@link Long}, {@link Float}, {@link Double},
     *        {@link String} or {@link org.ow2.util.scan.api.IType}.
     */
    @Override
    public void visit(final String name, final Object value) {
        if (ACTION.equals(name)) {
            javaxJwsWebMethod.setAction((String) value);
        } else if (EXCLUDE.equals(name)) {
            javaxJwsWebMethod.setExclude(((Boolean) value).booleanValue());
        } else if (OPERATION_NAME.equals(name)) {
            javaxJwsWebMethod.setOperationName((String) value);
        }
    }

    /**
     * Visits the end of the annotation. <br>
     * Creates the object and store it.
     */
    @Override
    public void visitEnd() {
        // add object
        getAnnotationMetadata().setJavaxJwsWebMethod(javaxJwsWebMethod);
    }


    /**
     * @return type of the annotation (its description).
     */
    public String getType() {
        return TYPE;
    }

}
